@@ -87,7 +87,7 @@ int get_fd_sockets(struct listen_endpoint *sockfd[])
     }
     if (sd > 0) {
       int i;
-      *sockfd = malloc(sd * sizeof(*sockfd[0]));
+      *sockfd = calloc(sd, sizeof(*sockfd[0]));
       CHECK_ALLOC(*sockfd, "malloc");
       for (i = 0; i < sd; i++) {
         (*sockfd)[i].socketfd = SD_LISTEN_FDS_START + i;
@@ -162,6 +162,17 @@ int listen_single_addr(struct addrinfo* addr, int keepalive, int udp)
     return sockfd;
 }
 
+void print_inet_listen_info(int fd, const char* addr, 
+                            struct sslhcfg_listen_item* cfg)
+{
+    print_message(msg_config, "%d:\t%s\t[%s] [%s] [max_cnx: %d]\n", 
+                  fd, 
+                  addr,
+                  cfg->keepalive ? "keepalive" : "",
+                  cfg->is_udp ? "udp" : "",
+                  cfg->max_connections);
+}
+
 
 /* Start listening internet sockets for configuration entry 'index' 
  * OUT: *sockfd[]: pointer to array of listen_endpoint object; we append new
@@ -182,12 +193,14 @@ static int start_listen_inet(struct listen_endpoint *sockfd[], int num_addr, str
     for (addr = start_addr; addr; addr = addr->ai_next) {
         num_addr++;
         *sockfd = realloc(*sockfd, num_addr * sizeof(*sockfd[0]));
+        memset(&(*sockfd)[num_addr-1], 0, sizeof((*sockfd)[num_addr-1]));
         (*sockfd)[num_addr-1].socketfd = listen_single_addr(addr, cfg->keepalive, cfg->is_udp);
         (*sockfd)[num_addr-1].type = cfg->is_udp ? SOCK_DGRAM : SOCK_STREAM;
         (*sockfd)[num_addr-1].family = AF_INET;
-        print_message(msg_config, "%d:\t%s\t[%s] [%s]\n", (*sockfd)[num_addr-1].socketfd, sprintaddr(buf, sizeof(buf), addr),
-                      cfg->keepalive ? "keepalive" : "",
-                      cfg->is_udp ? "udp" : "");
+        (*sockfd)[num_addr-1].endpoint_cfg = cfg;
+        print_inet_listen_info((*sockfd)[num_addr-1].socketfd, 
+                               sprintaddr(buf, sizeof(buf), addr), 
+                               cfg);
     }
     freeaddrinfo(start_addr);
     return num_addr;
@@ -216,9 +229,11 @@ static int start_listen_unix(struct listen_endpoint *sockfd[], int num_addr, str
 
     num_addr++;
     *sockfd = realloc(*sockfd, num_addr * sizeof(*sockfd[0]));
+    memset(&(*sockfd)[num_addr-1], 0, sizeof((*sockfd)[num_addr-1]));
     (*sockfd)[num_addr-1].socketfd = fd;
     (*sockfd)[num_addr-1].type = cfg->is_udp ? SOCK_DGRAM : SOCK_STREAM;
     (*sockfd)[num_addr-1].family = AF_INET;
+    (*sockfd)[num_addr-1].endpoint_cfg = cfg;
 
     return num_addr;
 }
@@ -499,10 +514,21 @@ void connect_addr(struct connection *cnx, int fd_from, connect_blocking blocking
     cnx->q[1].fd = fd;
 
     if (cnx->proto->proxyprotocol_is_present) {
-        pp_write_header(cnx->proto->proxyprotocol, cnx);
-        /* If pp_write_header() fails, it already logs a message and there is
-         * nothing much we can do. The server side will probably close the
-         * connection */
+        if (cnx->proto->proxyprotocol) {
+            pp_write_header(cnx->proto->proxyprotocol, cnx);
+            /* If pp_write_header() fails, it already logs a message and there is
+             * nothing much we can do. The server side will probably close the
+             * connection */
+        } else {
+            /* proxyprotocol == 0 means we remove a proxyprotocol header from
+             * the client-side, and log PP information (as it will be lost
+             * after sslh) */
+            int pp_len = pp_log_connection(cnx->q[1].begin_deferred_data,
+                                           cnx->q[1].deferred_data_size);
+            if (pp_len > 0) {
+                defer_skip(&cnx->q[1], pp_len);
+            }
+        }
     }
 }
 
@@ -525,6 +551,13 @@ int defer_write(struct queue *q, void* data, ssize_t data_size)
     return 0;
 }
 
+/* skip (remove) some data from the head of the queue */
+void defer_skip(struct queue *q, ssize_t data_size)
+{
+    q->deferred_data += data_size;
+    q->deferred_data_size -= data_size;
+}
+
 /* Store some data to write *before* what's already in the queue */
 int defer_write_before(struct queue *q, void* data, ssize_t data_size)
 {
@@ -595,6 +628,37 @@ void dump_connection(struct connection *cnx)
 }
 
 
+/* *cnx must have its proto field probed already.
+ * If required, increment the connection count for this protocol.
+ * Returns 1 if connection count is exceeded, 0 otherwise */
+int inc_proto_connections(struct sslhcfg_protocols_item* proto)
+{
+    proto->num_connections++;
+    if (proto->max_connections_is_present) {
+        print_message(msg_connections, "Proto %s +1: %d/%d cnx\n",
+                      proto->name,
+                      proto->num_connections,
+                      proto->max_connections);
+        if (proto->num_connections > proto->max_connections) {
+            print_message(msg_connections_error, "%s: too many connections, dropping\n", proto->name);
+            return 1;
+        }
+    }
+    return 0;
+}
+
+void dec_proto_connections(struct sslhcfg_protocols_item* proto)
+{
+    if (proto) {
+        proto->num_connections--;
+        print_message(msg_connections, "Proto %s -1: %d/%d cnx\n",
+                      proto->name,
+                      proto->num_connections,
+                      proto->max_connections);
+    }
+}
+
+
 /*
  * moves data from one fd to other
  *
@@ -849,16 +913,23 @@ int check_access_rights(int in_socket, const char* service)
     return 0;
 }
 
+volatile sig_atomic_t received_sigchld;
+
+void sig_sigchld(int sig)
+{
+    received_sigchld = 1;
+}
+
+
 void setup_signals(void)
 {
     int res;
     struct sigaction action;
 
-    /* Request no SIGCHLD is sent upon termination of
-     * the children */
+    /* Set SIGCHLD to sig_sigchld() */
     memset(&action, 0, sizeof(action));
-    action.sa_handler = NULL;
-    action.sa_flags = SA_NOCLDWAIT;
+    action.sa_handler = sig_sigchld;
+    action.sa_flags = SA_NOCLDSTOP;
     res = sigaction(SIGCHLD, &action, NULL);
     CHECK_RES_DIE(res, "sigaction");
 
@@ -873,7 +944,6 @@ void setup_signals(void)
     action.sa_handler = SIG_IGN;
     res = sigaction(SIGPIPE, &action, NULL);
     CHECK_RES_DIE(res, "sigaction");
-
 }
 
 
