@@ -30,20 +30,24 @@
 #include <pcre2.h>
 #endif
 
-#ifdef LIBBSD
-#include <bsd/unistd.h>
-#endif
-
 #include "common.h"
 #include "probe.h"
 #include "log.h"
 #include "tcp-probe.h"
 
+#if HAVE_LIBBSD
+#include <bsd/unistd.h>
+#endif
+
+#if HAVE_LIBCAP
+#include <sys/capability.h>
+#endif
+
 /* Constants for options that have no one-character shorthand */
 #define OPT_ONTIMEOUT   257
 
 static void printcaps(void) {
-#ifdef LIBCAP
+#if HAVE_LIBCAP
     cap_t caps;
     char* desc;
     ssize_t len;
@@ -67,23 +71,32 @@ static void printsettings(void)
     
     for (i = 0; i < cfg.protocols_len; i++ ) {
         p = &cfg.protocols[i];
-        strcpy(buf, "resolve on forward");
-        if (!p->resolve_on_forward) {
-            sprintaddr(buf, sizeof(buf), p->saddr);
-            size_t len = strlen(buf);
-            sprintf(buf+len, " family %d %d", 
-                    p->saddr->ai_family,
-                    p->saddr->ai_addr->sa_family);
+        if (p->is_unix) {
+            sprintf(buf, "unix socket: %s", p->host);
+        } else {
+            if (p->resolve_on_forward) {
+                strcpy(buf, "resolve on forward: ");
+                size_t len = strlen(buf);
+                sprintf(buf+len, "%s:%s", p->host, p->port);
+            } else {
+                sprintaddr(buf, sizeof(buf), p->saddr);
+                size_t len = strlen(buf);
+                sprintf(buf+len, " family %d %d",
+                        p->saddr->ai_family,
+                        p->saddr->ai_addr->sa_family);
+            }
         }
         print_message(msg_config, 
-                      "%s addr: %s. libwrap service: %s log_level: %d [%s] [%s] [%s]\n",
+                      "%s addr: %s. max_cnx: %d libwrap service: %s log_level: %d [%s] [%s] [%s] pp:%d\n",
                       p->name, 
                       buf,
+                      p->max_connections,
                       p->service,
                       p->log_level,
                       p->keepalive ? "keepalive" : "",
                       p->fork ? "fork" : "",
-                      p->transparent ? "transparent" : ""
+                      p->transparent ? "transparent" : "",
+                      p->proxyprotocol_is_present ? p->proxyprotocol : -1
                      );
     }
     print_message(msg_config, 
@@ -122,6 +135,7 @@ static void setup_regex_probe(struct sslhcfg_protocols_item *p)
             exit(1);
         }
     }
+    probe_regex_matches = pcre2_match_data_create(1, NULL);
 }
 #else
 {
@@ -129,6 +143,36 @@ static void setup_regex_probe(struct sslhcfg_protocols_item *p)
 }
 #endif
 
+/* Perform some fixups on configuration after reading it.
+ * if verbose is present, override all other verbose options
+ */
+void config_finish(struct sslhcfg_item* cfg)
+{
+    if (cfg->verbose) {
+        cfg->verbose_config = cfg->verbose;
+	cfg->verbose_config_error = cfg->verbose;
+	cfg->verbose_connections = cfg->verbose;
+	cfg->verbose_connections_try = cfg->verbose;
+	cfg->verbose_connections_error = cfg->verbose;
+	cfg->verbose_fd = cfg->verbose;
+	cfg->verbose_packets = cfg->verbose;
+	cfg->verbose_probe_info = cfg->verbose;
+	cfg->verbose_probe_error = cfg->verbose;
+	cfg->verbose_system_error = cfg->verbose;
+	cfg->verbose_int_error = cfg->verbose;
+    }
+}
+
+/* Checks that the UNIX socket specified exists and is accessible
+ * Dies otherwise
+ */
+static void check_access_unix_socket(struct sslhcfg_protocols_item* p)
+{
+    /* TODO */
+    return;
+}
+
+
 /* For each protocol in the configuration, resolve address and set up protocol
  * options if required
  */
@@ -138,7 +182,9 @@ static void config_protocols()
     for (i = 0; i < cfg.protocols_len; i++) {
         struct sslhcfg_protocols_item* p = &(cfg.protocols[i]);
 
-        if (
+        if (p->is_unix) {
+            check_access_unix_socket(p);
+        } else if (
             !p->resolve_on_forward &&
             resolve_split_name(&(p->saddr), p->host, p->port)
         ) {
@@ -219,22 +265,38 @@ void config_sanity_check(struct sslhcfg_item* cfg)
     }
 }
 
+/* Connect stdin, stdout, stderr to /dev/null. It is better to keep them around
+ * so they do not get re-used by socket descriptors, and accidently used by
+ * some library code.
+ */
+void close_std(void)
+{
+    int newfd;
+
+    if ((newfd = open("/dev/null", O_RDWR))) {
+        dup2 (newfd, STDIN_FILENO);
+        dup2 (newfd, STDOUT_FILENO);
+        dup2 (newfd, STDERR_FILENO);
+        /* close the helper handle, as this is now unnecessary */
+        close(newfd);
+    } else {
+        print_message(msg_system_error, "Error closing standard filehandles for background daemon\n");
+    }
+}
 
 int main(int argc, char *argv[], char* envp[])
 {
-
-   extern char *optarg;
-   extern int optind;
    int res, num_addr_listen;
    struct listen_endpoint *listen_sockets;
 
-#ifdef LIBBSD
+#if HAVE_LIBBSD
    setproctitle_init(argc, argv, envp);
 #endif
 
    memset(&cfg, 0, sizeof(cfg));
    res = sslhcfg_cl_parse(argc, argv, &cfg);
    if (res) exit(6);
+   config_finish(&cfg);
 
    if (cfg.version) {
        printf("%s %s\n", server_type, VERSION);
@@ -265,6 +327,7 @@ int main(int argc, char *argv[], char* envp[])
 
    if (!cfg.foreground) {
        if (fork() > 0) exit(0); /* Detach */
+       close_std();
 
        /* New session -- become group leader */
        if (getuid() == 0) {
